#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::OS::freebsd - FreeBSD specific configuration information
#
package Security::OS::freebsd;

use Exporter;
use Security::util::detaint;
use Security::util::die;
use Security::util::print;
use Security::util::read_config;
use strict;
use warnings "all";

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( init check );

our (%perms, $mtree, %pkgs, $pkg_info);
$mtree = '/usr/sbin/mtree';
$pkg_info = '/usr/sbin/pkg_info -ga';

sub init {
	print ( STDERR  "In Security::OS::freebsd::init()\n") if ( $main::opts{'d'} );

	&read_config('ignorepkgfiles');

	$main::group0 = 'wheel';
	$main::showmount = '/usr/bin/showmount';
	$main::exportsfile = '/etc/exports';
	$main::securefile='/etc/ttys';
	$main::accessfile = '/etc/login.access';
	$main::sudofile = '/usr/local/etc/sudoers';
	$main::kernelmodlist = '/sbin/kldstat | awk \' NR > 1 {print $NF}\'';
	# Find the directory where the loadable kernel modules are
	my $sysctl = '/usr/sbin/sysctl' if ( -f '/usr/sbin/sysctl' );
	$sysctl = '/sbin/sysctl' if ( -f '/sbin/sysctl' );
	if ( -f $sysctl ) {
		chop ($main::kerneldir=`$sysctl -n kern.module_path 2> /dev/null`);
		$main::kerneldir =~ s,^/;,^,o;
		$main::kerneldir =~ s/;/|^/go;
	}
}

sub check {
	print ( STDERR  "In Security::OS::freebsd::check()\n") if ( $main::opts{'d'} );

	&Security::rootlogin::check_access() 
		if ( defined &Security::rootlogin::check_access );

	&Security::rootlogin::check_ttys() 
		if ( defined &Security::rootlogin::check_ttys );

	&check_osfiles();

	&check_pkg_info();
}

sub check_pkg_info {
	my ($FP, $pkg, @files);

	print ( STDERR "In Security::OS::freebsd::check_pkg_info()\n") if ( $main::opts{'d'} );

	print ( STDERR "\tRunning cmd $pkg_info\n") if ( $main::opts{'d'} );
	open ($FP, $pkg_info . ' 2>&1 |') || &security_die ("Could not run $pkg_info\n");
	while (<$FP>) {
		chomp;

		$pkg=$1
			if ( /^Information for (.*):$/ );
		if (/^pkg_info: (.*) doesn't exist$/ && defined $pkg) {
			$pkgs{$1} = "file not found (package $pkg)"
				if ( !grep (/^\Q${1}\E$/, @{$main::config{'ignorepkgfiles'}}));
		}
		if (/^(.*) fails the original MD5 checksum$/ && defined $pkg) {
			$pkgs{$1} = "checksum mismatch (package $pkg)"
				if ( !grep (/^\Q${1}\E$/, @{$main::config{'ignorepkgfiles'}}));
		}
	}
	close $FP;
}

sub check_osfiles {
	my ($dir, $dir2, $file);
	my ($FP, $DIR);

	print ( STDERR "In Security::OS::freebsd::check_osfiles()\n") if ( $main::opts{'d'} );

	if ( opendir ($DIR, '/etc/mtree')) {  
		while ($file = &detaint('readdir mtree', readdir ($DIR))) {
			$dir = '';
			$dir = '/'  if ( $file =~ /BSD.root.dist/ );
			$dir = '/var'  if ( $file =~ /BSD.var.dist/ );
			$dir = '/usr'  if ( $file =~ /BSD.usr.dist/ );
			$dir = '/usr/include'
				if ( $file =~ /BSD.include.dist/ && -d '/usr/include' );
			$dir = '/usr/X11R6' 
				if ( $file =~ /BSD.x11.dist/ && -d '/usr/X11R6' );
			$dir = '/usr/local'
				if ( $file =~ /BSD.local.dist/ && -d '/usr/local' );
			if ( -d $dir && -f "/etc/mtree/$file" ) {
				print ( STDERR "\tRunning cmd $mtree -f /etc/mtree/$file -x -p $dir -e\n") if ( $main::opts{'d'} );
				open ($FP, "$mtree -f /etc/mtree/$file -x -p $dir -e|") || &security_die ("Could not run $mtree\n");
				while (<$FP>) {
					chomp;
					if ( length $_ > 1 ) {
						if ( /missing$/o ) {
							$dir2 = $_;
							$dir2 =~ s, missing$,,o;
							$dir2 =~ s,^\.,,o;
							if ( $dir ne '/' ) {
								$dir2 = (( $dir2 =~ m|^/|)
									? "$dir$dir2"
									: "$dir/$dir2");
							}
							$perms{$dir2} = "file not found"
								if ( !grep (/^\Q${dir2}\E$/, @{$main::config{'ignorepkgfiles'}}));
						} elsif ( /.* changed$/o ) {
							$dir2 = $_;
							$dir2 =~ s/ changed$//go;
							$dir2 =~ s,^\.,,o;
							# Make sure we don't get double // at the $dir $dir2 boundry
							$dir2 = (( $dir =~ m|/$| || $dir2 =~ m|^/| )
								? "$dir$dir2"
								: "$dir/$dir2");
							$perms{$dir2} = 1
								if ( !grep (/^\Q${dir2}\E$/, @{$main::config{'ignorepkgfiles'}}));
						} elsif ( exists $perms{$dir2} ) {
							$perms{$dir2}=($perms{$dir2} eq '1')
								? $_: $perms{$dir2} . $_;
							$perms{$dir2} =~ s/  / /go
								if ( !grep (/^\Q${dir2}\E$/, @{$main::config{'ignorepkgfiles'}}));
						}
					}
				}
				close $FP;
			}
		}
		closedir $DIR;
	}
}

sub print {
	print ( STDERR  "In Security::OS::freebsd::print()\n") if ( $main::opts{'d'} );

	&print_data ("\n\nchecking mtree command output:\n", %perms);

	&print_data ("\n\nchecking pkg_info command output:\n", %pkgs);
}

1

__END__

=head1 NAME

Security::OS::freebsd

=head1 SYNOPSIS

Security::OS::freebsd::init();

Security::OS::freebsd::check();

Security::OS::freebsd::print();

=head1 DESCRIPTION

Sets variables for OS specific commands.

=head1 API

Defines the following API calls:

=over 4

=item C<init()>

Init sets the following global variables:

=over 4

=item C<$main::showmount>

The location of the showmount command, used in the C<Security::exports> module.

=item C<$main::exportsfile>

The location of the exports file, used in C<Security::exports> module.

=item C<$main::sudofile>

The location of F<sudoers> file, used in C<Security::sudo> module.

=item C<$main::kerneldir>

The location of the kernel loadable modules, used in C<Security::kernel>
module.

=item C<$main::securefile>

The location of the list of ports root is allowed in on, used in
C<Security::rootlogin> module.

=back

=item C<check()>

Calls C<Security::rootlogin::check_ttys()> to check what root ports are
configured and compares it to what ports we know about.

Calls L<mtree(8)> command to verify what file metadata matches what the
OS thinks they should be.

Calls L<pkg_info(1)> command to find all the checksum mismatches and
missing files from the pkg_info database.

=item C<print()>

Prints out the L<mtree(8)> differences that we came across.

Prints out the L<pkg_info(1)> differences that we came across.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignorepkgfiles>

List of files to ignore errors returned from L<mtree(8)> and L<pkg_info(1)>.

=back

=head1 HISTORY

Checking the output of the L<pkg_info(1)> output came from the
FreeBSD security script in 8.2-RELEASE
(/etc/periodic/security/460.chkportsum).

=head1 SEE ALSO

L<mtree(8)>

L<sysctl(8)>

L<pkg_info(1)>

=cut
