#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::passwd - password file control module
#
package Security::passwd;

use Security::util::print;
use Security::util::read_config;
use Security::util::users;
use Exporter;
use strict;
use warnings "all";

our @ISA = qw(Exporter);
our @EXPORT = qw( init check print );

our (%expired, %badlogin, %baduid, %badgid, %uid0, %nopass, %dupuid, %defaultpw, %duppasswd, %oldpass);
our ($login_length) = 31;
our ($login_regex) = qr{^[\w\.-]+$};
our ($passwd_disable_regex) = qr{(^(x|NP|!+)$)|\*};
our ($id_max) = 4294967295;
our ($id_regex) = qr{^\d+$};

sub init {
	print ( STDERR  "In Security::passwd::init()\n") if ( $main::opts{'d'} );

	&read_config('ignorepasswd');
	&read_config('ignoreuid');
	&read_config('ignoredupuid');
	&read_config('ignoreduppasswd');
	&read_config('ignoreexpired');

	# If we have yet to read this in, then do it now (might be from Security::group::init()
	&read_config('defaultpass')
		if ( ! exists $main::config{'defaultpass'} || ! scalar @{$main::config{'defaultpass'}} );
}

sub check {
	print ( STDERR "In Security::password::check()\n") if ( $main::opts{'d'} );

	my ($oldname, $olduid);
	$olduid = -5;
	foreach my $name ( Security::util::users::get('name') ) {
		my $passwd = Security::util::users::get('name', $name, 'passwd');
		my $uid = Security::util::users::get('name', $name, 'uid');

		# Check to see who has UIDs of 0.  This comes from the security 
		# script from BSD 4.4.  We do not need to report the UID 0 on 
		# the 'root' account.
		$uid0{$name} = 1 
			if (! $uid && $name ne 'root' && 
				!grep (/^\Q${name}\E$/, @{$main::config{'ignoreuid'}}));

		# Check to see who has accounts without passwords.  This comes 
		# from the security script from BSD 4.4.
		$nopass{$name} = 1 if ($passwd eq '' &&
				!grep (/^\Q${name}\E$/, @{$main::config{'ignorepasswd'}}));

		# Check to see who has accounts with duplicate passwords.  
		if ( $passwd ne '' && $passwd !~ /$passwd_disable_regex/ ) {
			if ( exists $oldpass{$passwd} ) {
				$duppasswd{$name} = 'and ' . $oldpass{$passwd}
					if (!grep (/^\Q${name}\E$/, @{$main::config{'ignoreduppasswd'}}));
				$oldpass{$passwd} .= ' ' . $name
			} else {
				$oldpass{$passwd} = $name;
			}
		}

		# Check to see who has accounts with default passwords.  The
		# admin will define the list of default encrypted passwords in
		# the config file.
		$defaultpw{$name} = 1 if (ref($main::config{'defaultpass'}) eq 'ARRAY' &&
				grep (/^\Q${passwd}\E$/, @{$main::config{'defaultpass'}}));

		# Check to see what accounts have duplicate UIDs (except the 
		# root account, which was handled earlier.  This idea comes from 
		# the HP-UX System Security manual
		$dupuid{$name} = $oldname if ( $olduid eq $uid && $uid &&
				!grep (/^\Q${name}\E$/, @{$main::config{'ignoredupuid'}}));

		# Flag any accounts that have invalid field values.  This idea comes
		# from the OpenBSD 5.0 security script.
		$badlogin{$name} = 1
			if ( length $name > $login_length || $name !~ /$login_regex/ ); 

		# Flag [gu]id's that are negative, too large or otherwise not numeric
		# This idea comes from the OpenBSD 5.0 security script.
		$baduid{$name} = 1
			if ( $uid < 0 || $uid > $id_max || $uid !~ /$id_regex/ );
		my $gid = Security::util::users::get('name', $name, 'gid');
		$badgid{$name} = 1
			if ( $gid < 0 || $gid > $id_max || $gid !~ /$id_regex/ );

		# Flag any accounts that are expired.  This idea comes from the
		# OpenBSD 5.0 security script.
		my $expired = Security::util::users::get('name', $name, 'expire');
		$expired{$name} = 1
			if ( $expired &&
				!grep (/^\Q${name}\E$/, @{$main::config{'ignoreexpired'}}));

		$olduid = $uid;
		$oldname = $name;
	}
}

sub print () {
	print ( STDERR  "In Security::passwd::print()\n") if ( $main::opts{'d'} );

	&print_data ("\n\nchecking for invalid login names:\n", %badlogin);
	&print_data ("\n\nchecking for invalid UIDs:\n", %baduid);
	&print_data ("\n\nchecking for invalid GIDs (in passwd):\n", %badgid);
	&print_data ("\n\nchecking for UIDs of 0:\n", %uid0);
	&print_data ("\n\nchecking for accounts with no passwords:\n", %nopass);
	&print_data ("\n\nchecking for accounts with default passwords:\n", %defaultpw);
	&print_data ("\n\nchecking for duplicate UIDs:\n", %dupuid);
	&print_data ("\n\nchecking for duplicate passwords:\n", %duppasswd);
}

1;

__END__

=head1 NAME

Security::passwd - password file control module

=head1 SYNOPSIS

Security::passwd::init()

Security::passwd::check()

Security::passwd::print()

=head1 DESCRIPTION

Looks for accounts that have no password set.

Looks for accounts that have the same password, as comparing the hash
values.

Looks for accounts that have a known password, as compared to a list of
configured hashes.

Looks for accounts with duplicate UIDs.

Looks for accounts with bad login names.

Looks for accounts with bad UIDs.

Looks for accounts with bad GIDs.

Looks for expired accounts.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (ignorepasswd)> to get the list of accounts to ignore
that have no password.

Calls C<read_config (ignoreuid)> to get the list of accounts to ignore
that share UID 0.

Calls C<read_config (ignoredupuid)> to get the list of accounts to ignore
that share UIDs other than 0.

Calls C<read_config (ignoreduppasswd)> to get the list of accounts to ignore
that share passwords.

Calls C<read_config (ignoreexpired)> to get the list of accounts to ignore
that are marked as expired by the OS.

Calls C<read_config (defaultpass)> to get the list of password hashes that
we know of.

=item C<check()>

Traverses the accounts via C<Security::util::users::get()> looking for no
passwords, shared UIDs and passwords, known passwords, bad UIDs and GIDs,
bad login names and expired accounts.

=item C<print()>

Calls C<print_data ()> to print out the list of accounts other than root
that have UID 0.

Calls C<print_data ()> to print out the list of accounts that do not have
a password set.

Calls C<print_data ()> to print out the list of accounts that have a known
password.

Calls C<print_data ()> to print out the list of accounts that have duplicate
UIDs.

Calls C<print_data ()> to print out the list of accounts that have duplicate
passwords.

Calls C<print_data ()> to print out the list of accounts that have bad login
names.

Calls C<print_data ()> to print out the list of accounts that have bad UIDs.

Calls C<print_data ()> to print out the list of accounts that have bad GIDs.

Calls C<print_data ()> to print out the list of accounts that are expired.

=back 

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignorepasswd>

List of accounts to ignore if they have no password.  The C<sync> account
in SunOS 4.x is an example.

=item C<ignoreuid>

List of accounts to ignore that share UID 0.  C<root> and C<toor> accounts
on FreeBSD are examples.

=item C<ignoredupuid>

List of accounts to ignore that share NON 0 UIDs.

=item C<ignoreduppasswd>

List of accounts to ignore that share a password.

=item C<defaultpass>

List of password hashes to use as the list of known passwords.

=item C<ignoreexpired>

List of accounts to ignore that are marked as expired by the OS.

=back

=head1 HISTORY

The idea of looking for accounts without a password was suggested in
the BSD 4.4 security script.

The idea of looking for duplicate UID 0 accounts was suggested in the
BSD 4.4 security script.

The idea of looking for duplicate non UID 0 accounts was suggested
in the C<HP-UX System Security Manual>

The idea of looking for really long account names, negative or overly
large UIDs/GIDs, and expired accounts was suggested in the
OpenBSD 5.0 security script.

=head1 SEE ALSO

L<Security::util::users>

=cut

