#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::user - user config file control module
#
package Security::user;

use Security::util::print;
use Security::util::read_config;
use Security::util::stat;
use Security::util::users;
use Exporter;
use Fcntl qw(:mode);
use strict;
use warnings "all";

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( init check print );

our (%perms);

sub init {
	print ( STDERR  "In Security::user::init()\n") if ( $main::opts{'d'} );

	&read_config('userfiles');
	&read_config('ignoreusers');
}

sub check_file {
	my ($file, $uid, $mode) = @_;
	my (@res);

	# Use stat vs lstat, just in case someone has their
	# .forward file as a symlink to a different file.
	my ($dev,$ino,$m,$nlink,$u,$g,$rdev,$curbytes) = 
		&Security::util::stat::get_stat($file, 'stat');

	# Complain if the file is not owned by the user, unless it is owned by root
	push @res, "owner ($uid:$u)"
		if ( $u && $uid != $u );

	if ( ! defined $mode ) {

		# Found a group writable file
		push @res, 'group writable'
			if ( $m & &S_IWGRP() && !&S_ISDIR($m) );

		# Found a group writable directory (not sticky bit set either)
		push @res, 'group writable directory'
			if ( $m & &S_IWGRP() && &S_ISDIR($m) && !($m & 01000) );
			#if ( $m & &S_IWGRP() && &S_ISDIR($m) && !($m & &S_ISTXT()) );

		# Found a world writable file
		push @res, 'world writable'
			if ( $m & &S_IWOTH() && !&S_ISDIR($m) );

		# Found a world writable directory (not sticky bit set either)
		push @res, 'world writable directory'
			if ( $m & &S_IWOTH() && &S_ISDIR($m) && !($m & 01000) );
			#if ( $m & &S_IWOTH() && &S_ISDIR($m) && !($m & &S_ISTXT()) );
	} else {
		$mode = oct($mode);

		# Complain if the file is not the mode we want it to be
		push @res, sprintf ("mode (%4.4o:%4.4o)", $mode, ($m & 07777))
			if (($m & 07777) != $mode);
	}

	return (scalar @res
		? join (', ', @res)
		: undef);
}

sub check {
	print ( STDERR  "In Security::user::check()\n") if ( $main::opts{'d'} );

	foreach my $name ( Security::util::users::get('name') ) {
		my $dir = Security::util::users::get('name', $name, 'dir');
		my $uid = Security::util::users::get('name', $name, 'uid');

		if ( !grep (/^\Q$name\E/, @{$main::config{'ignoreusers'}}) && -d $dir ) {

			# First check the ownership and permissions of the home directory.
			my $res = &check_file($dir, $uid, undef);
			$perms{$dir} = $res if ( defined $res );

			foreach my $f ( @{$main::config{'userfiles'}} ) {
				my ($file, $mode) = split (':', $f);
				if ( -f $dir . '/' . $file ) {

					$res = &check_file($dir . '/' . $file, $uid, $mode);
					$perms{$dir . '/' . $file} = $res if ( defined $res );

				}
			}
		}
	}
}

sub print {
	print ( STDERR  "In Security::user::print()\n") if ( $main::opts{'d'} );

	&print_data ("\n\nchecking for user files with improper settings:\n", %perms);
}

1;

__END__

=head1 NAME

Security::user - user config file check

=head1 SYNOPSIS

Security::user::init()

Security::user::check()

Security::user::print()

=head1 DESCRIPTION

Checks the file modes and ownership of user config files.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (useronlyfiles)> to get the list of files to look for
in a user's home directory.

Calls C<read_config (userfiles)> to get the list of files to look for
in a user's home directory.

Calls C<read_config (ignoreusers)> to get the list of accounts to
skip checking for the userfiles.

=item C<check()>

Goes through each account on the system (via C<Security::util::users-:get()>),
and check to see if they have any file listed in C<userfiles>).
If so, then call C<Security::util::stat::get_stat($file, 'stat')> to check the
owner and mode of the file.  It will also check to see if the home directory
has the correct ownership and mode.

=item C<print()>

Calls C<print_data ()> to print out the list of user config files with the wrong
owner.

Calls C<print_data ()> to print out the list of user config files that are world
writable.

Calls C<print_data ()> to print out the list of user config files that are group
writable.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<useronlyfiles>

The list of files to check to make sure the owner, group and mode are set
correctly.  The mode should be set so only the user can read/write the files.

=item C<userfiles>

The list of files to check to make sure the owner, group and mode are set
correctly.  The mode should be set so only the user can write the file.

=item C<ignoreusers>

The list of accounts to ignore problems with user config files.  Usually is the list
of accounts that have the same home directory, most commonly F</>.

=back

=head1 HISTORY

This module used to be called Security::mail and the idea came from error messages
from C<sendmail(1)>.  The OpenBSD 5.0 Security script was the source of the idea to
make this routine more generic to test other user configuration files.

=head1 SEE ALSO

L<Security::util::stat>

L<Security::util::users>

L<sendmail(1)>

L<procmail(1)>

L<foward(5)>

=cut
