#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::kernel - loadable kernel modules control module
#
package Security::kernel;

use Security::util::print;
use Security::util::die;
use Security::util::read_config;
use Security::find;
use Security::util::stat;
use Security::db;
use Exporter;
use POSIX;
use strict;
use warnings "all";

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( init check print );

our (%kernelperms, %kernelload);

sub init {
	print ( STDERR  "In Security::kernel::init()\n") if ( $main::opts{'d'} );

	&read_config('kernelmodules');

	&Security::find::add_wanted (\&kernel_wanted);
}

# Added $kerneldir checks based on the SANS98 Conference
sub kernel_wanted {
	my ($file) = @_;
	if ( ! -l $file && -f $file ) {
		if ( defined $main::kerneldir && 
			$File::Find::name =~ /^$main::kerneldir/ ) {

			my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$curbytes) = 
				&Security::util::stat::get_stat($File::Find::name, 'lstat');

			&Security::db::verify($File::Find::name);

			# If the file is world writeable, then flag this case too
			$kernelperms{$File::Find::name} = 1
				if ( defined $mode && $mode & &S_IWOTH() );
		}
	}
}

sub check {
	my ($FP);

	print ( STDERR  "In Security::kernel::check()\n") if ( $main::opts{'d'} );

	# Don't do anything if we don't have a list of valid kernel modules
	return if ( ! scalar @{$main::config{'kernelmodules'}} );

	print ( STDERR "\tRunning cmd $main::kernelmodlist\n") if ( $main::opts{'d'} );
	open($FP, "$main::kernelmodlist |") || &security_die("Could not run kernel module listing program ($main::kernelmodlist)\n");
	while(<$FP>) {
		chomp;
		my $module = $_;
		$kernelload{$module} = 1
			if ( !grep (/^\Q${module}\E$/, @{$main::config{'kernelmodules'}}));
	}
	close $FP;
}

sub print {
	print ( STDERR  "In Security::kernel::print()\n") if ( $main::opts{'d'} );

	&print_data ("\n\nchecking for world writable kernel modules:\n", %kernelperms);
	&print_data ("\n\nchecking for unexpectedly loaded kernel modules:\n", %kernelload);
}

1;

__END__

=head1 NAME

Security::kernel - kernel loadable module file checks

=head1 SYNOPSIS

Security::kernel::init()

Security::kernel::check()

Security::kernel::print()

=head1 DESCRIPTION

Checks the checksum and file mode of any file listed under
C<$main::kerneldir>.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<add_wanted (\&kernel_wanted)> to implement the list of directories
where will find kernel loadable modules.

=item C<check()>

Checks the list of currently loaded kernel modules (as defined by
C<$main::kernelmodlist>) against the C<kernelmodules> defined in
the config file.

=item C<print()>

Calls C<print_data ()> to print out the list of kernel modules that were
world writable.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<kernelmodules>

List of kernel modules to find loaded on the system.  If a kernel module
is loaded and not on this list, report the error.

=back

=head1 HISTORY

The idea for the checksuming and checking for world writeable kernel modules
came from the SANS 1998 Security conference in Orlando, FL.  The idea to check
the currently loaded kernel modules was an offshoot of the 2011 kernel.org
exploit.

=head1 TODO

Convert C<$main::kerneldir> to C<kerneldirs>.

=head1 SEE ALSO

L<Security::find>

L<Security::util::stat>

L<Security::db>

=cut

